/**
 * @license
 * Copyright (C) 2012 Pyrios
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview
 * Registers a language handler for TCL
 *
 *
 * To use, include prettify.js and this file in your HTML page.
 * Then put your code in an HTML tag like
 *      <pre class="prettyprint lang-tcl">proc foo {} {puts bar}</pre>
 *
 * I copy-pasted lang-lisp.js, so this is probably not 100% accurate.
 * I used http://wiki.tcl.tk/1019 for the keywords, but tried to only
 * include as keywords that had more impact on the program flow
 * rather than providing convenience. For example, I included 'if'
 * since that provides branching, but left off 'open' since that is more
 * like a proc. Add more if it makes sense.
 *
 * @author pyrios@gmail.com
 */

registerLangHandler(
    createSimpleLexer(
        [
         ['opn',             RegExpProxy( "^\\{+" ), null, '{'],
         ['clo',             RegExpProxy( "^\\}+" ), null, '}'],
         // A line comment that starts with ;
         [PR_COMMENT,     RegExpProxy( "^#[^\\r\\n]*" ), null, '#'],
         // Whitespace
         [PR_PLAIN,       RegExpProxy( "^[\\t\\n\\r \\xA0]+" ), null, '\t\n\r \xA0'],
         // A double quoted, possibly multi-line, string.
         [PR_STRING,      RegExpProxy( "^\\\"(?:[^\\\"\\\\]|\\\\[\\s\\S])*(?:\\\"|$)" ), null, '"']
        ],
        [
         [PR_KEYWORD,     RegExpProxy( "^(?:after|append|apply|array|break|case|catch|continue|error|eval|exec|exit|expr|for|foreach|if|incr|info|proc|return|set|switch|trace|uplevel|upvar|while)\\b" ), null],
         [PR_LITERAL,
          RegExpProxy( "^[+\\-]?(?:[0#]x[0-9a-f]+|\\d+\\/\\d+|(?:\\.\\d+|\\d+(?:\\.\\d*)?)(?:[ed][+\\-]?\\d+)?)", 'i' )],
         // A single quote possibly followed by a word that optionally ends with
         // = ! or ?.
         [PR_LITERAL,
          RegExpProxy( "^\\'(?:-*(?:\\w|\\\\[\\x21-\\x7e])(?:[\\w-]*|\\\\[\\x21-\\x7e])[=!?]?)?" )],
         // A word that optionally ends with = ! or ?.
         [PR_PLAIN,
          RegExpProxy( "^-*(?:[a-z_]|\\\\[\\x21-\\x7e])(?:[\\w-]*|\\\\[\\x21-\\x7e])[=!?]?", 'i' )],
         // A printable non-space non-special character
         [PR_PUNCTUATION, RegExpProxy( "^[^\\w\\t\\n\\r \\xA0()\\\"\\\\\\';]+" )]
        ]),
    ['tcl']);
